@@ -8,70 +8,36 @@ except ImportError:
     import json
 
 from flask import Blueprint, render_template, abort, request, url_for, redirect, flash, Response
-from flask_peewee.forms import CustomModelConverter
-from flask_peewee.filters import QueryFilter, lookups_for_field, Lookup
+from flask_peewee.filters import FilterMapping, FilterForm, FilterModelConverter
+from flask_peewee.forms import BaseModelConverter
 from flask_peewee.serializer import Serializer
 from flask_peewee.utils import get_next, PaginatedQuery, path_to_models, slugify
-from peewee import BooleanField, DateTimeField, ForeignKeyField, DateField
+from peewee import BooleanField, DateTimeField, ForeignKeyField, DateField, TextField
 from werkzeug import Headers
 from wtforms import fields, widgets
-from wtfpeewee.fields import ModelSelectField, ModelSelectMultipleField
+from wtfpeewee.fields import ModelSelectField, ModelSelectMultipleField, ModelHiddenField
 from wtfpeewee.orm import model_form
 
 
 current_dir = os.path.dirname(__file__)
 
 
-class FieldValueMap(object):
-    def __init__(self, model_admin):
+class AdminModelConverter(BaseModelConverter):
+    def __init__(self, model_admin, additional=None):
+        super(AdminModelConverter, self).__init__(additional)
         self.model_admin = model_admin
 
-    def get_conversions(self):
-        return {
-            (BooleanField, 'eq'): self.handle_boolean,
-            (DateTimeField, 'today'): self.handle_dummy,
-            (DateTimeField, 'yesterday'): self.handle_dummy,
-            (DateTimeField, 'this_week'): self.handle_dummy,
-            (DateField, 'today'): self.handle_dummy,
-            (DateField, 'yesterday'): self.handle_dummy,
-            (DateField, 'this_week'): self.handle_dummy,
-            (ForeignKeyField, 'eq'): self.handle_foreign_key,
-            (ForeignKeyField, 'in'): self.handle_foreign_key_multi,
-        }
+        self.converters[ForeignKeyField] = self.handle_foreign_key
 
-    def convert(self, field, lookup, lookup_name, prefix):
-        conversions = self.get_conversions()
-        key = (type(field), lookup)
-        lookup_obj = conversions.get(key, self.handle_default)(field, lookup, prefix)
+    def handle_foreign_key(self, model, field, **kwargs):
+        if field.null:
+            kwargs['allow_blank'] = True
 
-        # wat
-        lookup_obj.lookup_name = lookup_name
-        lookup_obj.css_class = 'span2 input-small lookup-input'
-        return lookup_obj
-
-    def handle_default(self, field, lookup, prefix):
-        return Lookup(field, lookup, 'text', prefix=prefix)
-
-    def handle_dummy(self, field, lookup, prefix):
-        return Lookup(field, lookup, 'hidden', prefix=prefix)
-
-    def handle_boolean(self, field, lookup, prefix):
-        return Lookup(field, lookup, 'select', [('1', 'True'), ('', 'False')], prefix=prefix)
-
-    def fk_to_list(self, field):
-        return [(obj.get_pk(), unicode(obj)) for obj in field.to.select()]
-
-    def handle_foreign_key(self, field, lookup, prefix):
-        _fkl = self.model_admin.foreign_key_lookups or {}
-        if field.name not in _fkl:
-            return Lookup(field, lookup, 'select', self.fk_to_list(field), prefix=prefix)
-        return Lookup(field, lookup, 'foreign_key', field.to, prefix=prefix)
-
-    def handle_foreign_key_multi(self, field, lookup, prefix):
-        _fkl = self.model_admin.foreign_key_lookups or {}
-        if field.name not in _fkl:
-            return Lookup(field, lookup, 'select_multiple', self.fk_to_list(field), prefix=prefix)
-        return Lookup(field, lookup, 'foreign_key_multiple', field.to, prefix=prefix)
+        if field.name in (self.model_admin.foreign_key_lookups or ()):
+            form_field = ModelHiddenField(model=field.rel_model, **kwargs)
+        else:
+            form_field = ModelSelectField(model=field.rel_model, **kwargs)
+        return field.name, form_field
 
 
 class ModelAdmin(object):
@@ -85,14 +51,17 @@ class ModelAdmin(object):
     # a model instance, though in the latter case they will not be sortable
     columns = None
 
-    # filter parameters -- what is available for filtering via the request
-    exclude_filters = None
-    ignore_filters = ('ordering', 'page',)
+    # exclude certian fields from being exposed as filters -- for related fields
+    # use "__" notation, e.g. user__password
+    filter_exclude = None
+    filter_fields = None
 
     # form parameters, lists of fields
     exclude = None
     fields = None
 
+    form_converter = AdminModelConverter
+
     # foreign_key_field --> related field to search on, e.g. {'user': 'username'}
     foreign_key_lookups = None
 
@@ -100,6 +69,9 @@ class ModelAdmin(object):
     delete_collect_objects = True
     delete_recursive = True
 
+    filter_mapping = FilterMapping
+    filter_converter = FilterModelConverter
+
     # templates, to override see get_template_overrides()
     base_templates = {
         'index': 'admin/models/index.html',
@@ -112,7 +84,8 @@ class ModelAdmin(object):
     def __init__(self, admin, model):
         self.admin = admin
         self.model = model
-        self.pk_name = self.model._meta.pk_name
+        self.db = model._meta.database
+        self.pk = self.model._meta.primary_key
 
         self.templates = dict(self.base_templates)
         self.templates.update(self.get_template_overrides())
@@ -127,9 +100,28 @@ class ModelAdmin(object):
             name,
         )
 
+    def get_filter_form(self):
+        return FilterForm(
+            self.model,
+            self.filter_converter(),
+            self.filter_mapping(),
+            self.filter_fields,
+            self.filter_exclude,
+        )
+
+    def process_filters(self, query):
+        filter_form = self.get_filter_form()
+        form, query, cleaned = filter_form.process_request(query)
+        return form, query, cleaned, filter_form._field_tree
+
     def get_form(self, adding=False):
         allow_pk = adding and not self.model._meta.auto_increment
-        return model_form(self.model, allow_pk=allow_pk, only=self.fields, exclude=self.exclude, converter=CustomModelConverter(self))
+        return model_form(self.model,
+            allow_pk=allow_pk,
+            only=self.fields,
+            exclude=self.exclude,
+            converter=self.form_converter(self),
+        )
 
     def get_add_form(self):
         return self.get_form(adding=True)
@@ -141,10 +133,7 @@ class ModelAdmin(object):
         return self.model.select()
 
     def get_object(self, pk):
-        return self.get_query().get(**{self.pk_name: pk})
-
-    def get_query_filter(self, query):
-        return QueryFilter(query, self.ignore_filters)
+        return self.get_query().where(self.pk==pk).get()
 
     def get_urls(self):
         return (
@@ -168,37 +157,6 @@ class ModelAdmin(object):
     def get_admin_name(self):
         return slugify(self.model.__name__)
 
-    def get_lookups(self, prefix='', seen=None):
-        field_value_map = FieldValueMap(self)
-
-        seen = seen or set()
-        lookups = {}
-        active_lookups = []
-
-        for field in self.model._meta.get_fields():
-            if self.exclude_filters and field.name in self.exclude_filters:
-                continue
-
-            for lookup, lookup_name in lookups_for_field(field):
-                key = (prefix, field)
-                lookups.setdefault(key, [])
-                lookup_obj = field_value_map.convert(field, lookup, lookup_name, prefix)
-                lookups[key].append(lookup_obj)
-
-                if lookup_obj.name in request.args:
-                    active_lookups.append(lookup_obj)
-
-            if isinstance(field, ForeignKeyField):
-                rel_prefix = '%s%s__' % (prefix, field.name)
-                rel_model = field.to
-                if rel_model in self.admin and (rel_model, field.name) not in seen:
-                    seen.add((rel_model, field.name))
-                    rel_lookups, rel_active = self.admin[rel_model].get_lookups(rel_prefix, seen)
-                    lookups.update(rel_lookups)
-                    active_lookups.extend(rel_active)
-
-        return lookups, active_lookups
-
     def save_model(self, instance, form, adding=False):
         form.populate_obj(instance)
         instance.save(force_insert=adding)
@@ -208,7 +166,8 @@ class ModelAdmin(object):
         if ordering:
             desc, column = ordering.startswith('-'), ordering.lstrip('-')
             if self.column_is_sortable(column):
-                query = query.order_by((column, desc and 'desc' or 'asc'))
+                field = self.model._meta.fields[column]
+                query = query.order_by(field.asc() if not desc else field.desc())
         return query
 
     def index(self):
@@ -217,31 +176,26 @@ class ModelAdmin(object):
         ordering = request.args.get('ordering') or ''
         query = self.apply_ordering(query, ordering)
 
-        # create a QueryFilter object with our current query
-        query_filter = self.get_query_filter(query)
-
         # process the filters from the request
-        filtered_query = query_filter.get_filtered_query()
+        filter_form, query, cleaned, field_tree = self.process_filters(query)
 
         # create a paginated query out of our filtered results
-        pq = PaginatedQuery(filtered_query, self.paginate_by)
+        pq = PaginatedQuery(query, self.paginate_by)
 
         if request.method == 'POST':
             id_list = request.form.getlist('id')
             if request.form['action'] == 'delete':
                 return redirect(url_for(self.get_url_name('delete'), id=id_list))
             else:
-                return redirect(url_for(self.get_url_name('export'), id__in=id_list))
-
-        lookups, active_lookups = self.get_lookups()
+                return redirect(url_for(self.get_url_name('export'), id=id_list))
 
         return render_template(self.templates['index'],
             model_admin=self,
             query=pq,
             ordering=ordering,
-            query_filter=query_filter,
-            lookups=lookups,
-            active_lookups=active_lookups,
+            filter_form=filter_form,
+            field_tree=field_tree,
+            active_filters=cleaned,
         )
 
     def dispatch_save_redirect(self, instance):
@@ -251,7 +205,7 @@ class ModelAdmin(object):
             return redirect(url_for(self.get_url_name('add')))
         else:
             return redirect(
-                url_for(self.get_url_name('edit'), pk=instance.get_pk())
+                url_for(self.get_url_name('edit'), pk=instance.get_id())
             )
 
     def add(self):
@@ -288,15 +242,15 @@ class ModelAdmin(object):
         return render_template(self.templates['edit'], model_admin=self, instance=instance, form=form)
 
     def collect_objects(self, obj):
-        select_queries, nullable_queries = obj.collect_queries()
+        deps = obj.dependencies()
         objects = []
 
-        for query, fk_field, depth in select_queries:
-            model = query.model
-            query.query = ['*']
-            collected = [obj for obj in query.execute().iterator()]
-            if collected:
-                objects.append((depth, model, fk_field, collected))
+        for query, fk in obj.dependencies():
+            if not fk.null:
+                sq = fk.model_class.select().where(query)
+                collected = [rel_obj for rel_obj in sq.execute().iterator()]
+                if collected:
+                    objects.append((0, fk.model_class, collected))
 
         return sorted(objects, key=lambda i: (i[0], i[1].__name__))
 
@@ -306,15 +260,13 @@ class ModelAdmin(object):
         else:
             id_list = request.form.getlist('id')
 
-        query = self.model.select().where(**{
-            '%s__in' % self.model._meta.pk_name: id_list
-        })
+        query = self.model.select().where(self.pk << id_list)
 
         if request.method == 'GET':
             collected = {}
             if self.delete_collect_objects:
                 for obj in query:
-                    collected[obj.get_pk()] = self.collect_objects(obj)
+                    collected[obj.get_id()] = self.collect_objects(obj)
 
         elif request.method == 'POST':
             count = query.count()
@@ -334,7 +286,7 @@ class ModelAdmin(object):
         path_str = '__'.join(path)
         for field in model._meta.get_fields():
             if isinstance(field, ForeignKeyField):
-                self.collect_related_fields(field.to, accum, path + [field.name])
+                self.collect_related_fields(field.rel_model, accum, path + [field.name])
             elif model != self.model:
                 accum.setdefault((model, path_str), [])
                 accum[(model, path_str)].append(field)
@@ -347,29 +299,24 @@ class ModelAdmin(object):
         ordering = request.args.get('ordering') or ''
         query = self.apply_ordering(query, ordering)
 
-        # create a QueryFilter object with our current query
-        query_filter = self.get_query_filter(query)
-
         # process the filters from the request
-        filtered_query = query_filter.get_filtered_query()
-
+        filter_form, query, cleaned, field_tree = self.process_filters(query)
         related = self.collect_related_fields(self.model, {}, [])
 
         if request.method == 'POST':
             raw_fields = request.form.getlist('fields')
-            export = Export(filtered_query, related, raw_fields)
+            export = Export(query, related, raw_fields)
             return export.json_response('export-%s.json' % self.get_admin_name())
 
-        lookups, active_lookups = self.get_lookups()
-
         return render_template(self.templates['export'],
             model_admin=self,
-            model=filtered_query.model,
-            query=filtered_query,
-            query_filter=query_filter,
+            model=query.model_class,
+            query=query,
+            filter_form=filter_form,
+            field_tree=field_tree,
+            active_filters=cleaned,
             related_fields=related,
-            lookups=lookups,
-            active_lookups=active_lookups,
+            sql=query.sql(self.db.get_compiler()),
         )
 
     def ajax_list(self):
@@ -383,11 +330,11 @@ class ModelAdmin(object):
             data = []
         else:
             rel_model = models.pop()
-            rel_field = self.foreign_key_lookups[field]
-
-            query = rel_model.select().where(**{
-                '%s__icontains'% rel_field: request.args.get('query', ''),
-            }).order_by(rel_field)
+            rel_field = rel_model._meta.fields[self.foreign_key_lookups[field]]
+            query = rel_model.select().order_by(rel_field)
+            query_string = request.args.get('query')
+            if query_string:
+                query = query.where(rel_field ** ('%%%s%%' % query_string))
 
             pq = PaginatedQuery(query, self.filter_paginate_by)
             current_page = pq.get_page()
@@ -397,7 +344,7 @@ class ModelAdmin(object):
                 next_page = current_page + 1
 
             data = [
-                {'id': obj.get_pk(), 'repr': unicode(obj)} \
+                {'id': obj.get_id(), 'repr': unicode(obj)} \
                     for obj in pq.get_list()
             ]
 
@@ -474,7 +421,7 @@ class AdminTemplateHelper(object):
     def get_admin_url(self, obj):
         model_admin = self.admin.get_admin_for(type(obj))
         if model_admin:
-            return url_for(model_admin.get_url_name('edit'), pk=obj.get_pk())
+            return url_for(model_admin.get_url_name('edit'), pk=obj.get_id())
 
     def get_model_name(self, model_class):
         model_admin = self.admin.get_admin_for(model_class)
@@ -482,6 +429,13 @@ class AdminTemplateHelper(object):
             return model_admin.get_display_name()
         return model_class.__name__
 
+    def apply_prefix(self, field_name, prefix_accum, field_prefix, rel_prefix='fr_', rel_sep='-'):
+        accum = []
+        for prefix in prefix_accum:
+            accum.append('%s%s' % (rel_prefix, prefix))
+        accum.append('%s%s' % (field_prefix, field_name))
+        return rel_sep.join(accum)
+
     def prepare_environment(self):
         self.app.template_context_processors[None].append(self.get_model_admins)
 
@@ -493,6 +447,8 @@ class AdminTemplateHelper(object):
         self.app.jinja_env.globals['get_admin_url'] = self.get_admin_url
         self.app.jinja_env.globals['get_model_name'] = self.get_model_name
 
+        self.app.jinja_env.filters['apply_prefix'] = self.apply_prefix
+
 
 class Admin(object):
     def __init__(self, app, auth, template_helper=AdminTemplateHelper,
@@ -626,7 +582,7 @@ class Export(object):
     def prepare_query(self):
         clone = self.query.clone()
 
-        select = {}
+        select = []
 
         def ensure_join(query, m, p):
             if m not in query._joined_models:
@@ -651,10 +607,10 @@ class Export(object):
                 model = self.query.model
                 column = lookup
 
-            select.setdefault(model, [])
-            select[model].append(column)
+            field = model._meta.fields[column]
+            select.append(field)
 
-        clone.query = select
+        clone._select = select
         return clone
 
     def json_response(self, filename='export.json'):
