@@ -1,4 +1,5 @@
 import functools
+import operator
 try:
     import simplejson as json
 except ImportError:
@@ -6,8 +7,9 @@ except ImportError:
 
 from flask import Blueprint, abort, request, Response, session, redirect, url_for, g
 from peewee import *
+from peewee import DJANGO_MAP
 
-from flask_peewee.filters import QueryFilter
+from flask_peewee.filters import make_field_tree
 from flask_peewee.serializer import Serializer, Deserializer
 from flask_peewee.utils import PaginatedQuery, slugify, get_object_or_404
 
@@ -31,16 +33,24 @@ class APIKeyAuthentication(Authentication):
     Requires a model that has at least two fields, "key" and "secret", which will
     be searched for when authing a request
     """
+    key_field = 'key'
+    secret_field = 'secret'
+
     def __init__(self, model, protected_methods=None):
         super(APIKeyAuthentication, self).__init__(protected_methods)
         self.model = model
+        self._key_field = model._meta.fields[self.key_field]
+        self._secret_field = model._meta.fields[self.secret_field]
 
     def get_query(self):
         return self.model.select()
 
     def get_key(self, k, s):
         try:
-            return self.get_query().get(key=k, secret=s)
+            return self.get_query().where(
+                self._key_field==k,
+                self._secret_field==s
+            ).get()
         except self.model.DoesNotExist:
             pass
 
@@ -90,14 +100,16 @@ class AdminAuthentication(UserAuthentication):
 class RestResource(object):
     paginate_by = 20
 
-    # filtering
-    exclude_filters = None
-    ignore_filters = ('ordering', 'page', 'limit', 'key', 'secret',)
-
     # serializing: dictionary of model -> field names to restrict output
     fields = None
     exclude = None
 
+    # exclude certian fields from being exposed as filters -- for related fields
+    # use "__" notation, e.g. user__password
+    filter_exclude = None
+    filter_fields = None
+    filter_recursive = True
+
     # mapping of field name to resource class
     include_resources = None
 
@@ -107,6 +119,8 @@ class RestResource(object):
     def __init__(self, rest_api, model, authentication, allowed_methods=None):
         self.api = rest_api
         self.model = model
+        self.pk = model._meta.primary_key
+
         self.authentication = authentication
         self.allowed_methods = allowed_methods or ['GET', 'POST', 'PUT', 'DELETE']
 
@@ -116,19 +130,29 @@ class RestResource(object):
         else:
             self._exclude = {}
 
+        self._filter_fields = self.filter_fields or self.model._meta.get_field_names()
+        self._filter_exclude = self.filter_exclude or []
+
         self._resources = {}
+
+        # recurse into nested resources
         if self.include_resources:
             for field_name, resource in self.include_resources.items():
                 field_obj = self.model._meta.fields[field_name]
-                resource_obj = resource(self.api, field_obj.to, self.authentication, self.allowed_methods)
+                resource_obj = resource(self.api, field_obj.rel_model, self.authentication, self.allowed_methods)
                 self._resources[field_name] = resource_obj
                 self._fields.update(resource_obj._fields)
                 self._exclude.update(resource_obj._exclude)
 
+                self._filter_fields.extend(['%s__%s' % (field_name, ff) for ff in resource_obj._filter_fields])
+                self._filter_exclude.extend(['%s__%s' % (field_name, ff) for ff in resource_obj._filter_exclude])
+
             self._include_foreign_keys = False
         else:
             self._include_foreign_keys = True
 
+        self._field_tree = make_field_tree(self.model, self._filter_fields, self._filter_exclude, self.filter_recursive)
+
     def authorize(self):
         return self.authentication.authorize()
 
@@ -145,8 +169,48 @@ class RestResource(object):
     def get_query(self):
         return self.model.select()
 
-    def get_query_filter(self, query):
-        return QueryFilter(query, self.exclude_filters, self.ignore_filters)
+    def process_query(self, query):
+        raw_filters = {}
+
+        # clean and normalize the request parameters
+        for key in request.args:
+            if '__' in key:
+                expr, op = key.rsplit('__', 1)
+                if op not in DJANGO_MAP:
+                    expr = key
+                    op = 'eq'
+            else:
+                expr = key
+                op = 'eq'
+            raw_filters.setdefault(expr, [])
+            raw_filters[expr].append((op, request.args.getlist(key)))
+
+        # do a breadth first search across the field tree created by filter_fields,
+        # searching for matching keys in the request parameters -- when found,
+        # filter the query accordingly
+        queue = [(self._field_tree, '')]
+        while queue:
+            node, prefix = queue.pop(0)
+            for field in node.fields:
+                filter_expr = '%s%s' % (prefix, field.name)
+                if filter_expr in raw_filters:
+                    for op, arg_list in raw_filters[filter_expr]:
+                        query = self.apply_filter(query, filter_expr, op, arg_list)
+
+            for child_prefix, child_node in node.children.items():
+                queue.append((child_node, prefix + child_prefix + '__'))
+
+        return query
+
+    def apply_filter(self, query, expr, op, arg_list):
+        query_expr = '%s__%s' % (expr, op)
+        if op == 'in':
+            return query.filter(**{query_expr: arg_list})
+        elif len(arg_list) == 1:
+            return query.filter(**{query_expr: arg_list[0]})
+        else:
+            query_clauses = [DQ(**{query_expr: val}) for val in arg_list]
+            return query.filter(reduce(operator.or_, query_clauses))
 
     def get_serializer(self):
         return Serializer()
@@ -231,9 +295,7 @@ class RestResource(object):
             return self.create()
 
     def api_detail(self, pk, method=None):
-        obj = get_object_or_404(self.get_query(), **{
-            self.model._meta.pk_name: pk
-        })
+        obj = get_object_or_404(self.get_query(), self.pk==pk)
 
         method = method or request.method
 
@@ -255,7 +317,8 @@ class RestResource(object):
         if ordering:
             desc, column = ordering.startswith('-'), ordering.lstrip('-')
             if column in self.model._meta.fields:
-                query = query.order_by((column, desc and 'desc' or 'asc'))
+                field = self.model._meta.fields[column]
+                query = query.order_by(field.asc() if not desc else field.desc())
 
         return query
 
@@ -303,16 +366,13 @@ class RestResource(object):
         query = self.get_query()
         query = self.apply_ordering(query)
 
-        # create a QueryFilter object with our current query
-        query_filter = self.get_query_filter(query)
-
-        # process the filters from the request
-        filtered_query = query_filter.get_filtered_query()
+        # process any filters
+        query = self.process_query(query)
 
         if self.paginate_by or 'limit' in request.args:
-            return self.paginated_object_list(filtered_query)
+            return self.paginated_object_list(query)
 
-        return self.response(self.serialize_query(filtered_query))
+        return self.response(self.serialize_query(query))
 
     def object_detail(self, obj):
         return self.response(self.serialize_object(obj))
